#import "context_tests.h"


@implementation SGContextTests

- (void)setUp
{
	[super setUp];

	if (!_client) {
		NSString *u = @"quicoteto";
		NSString *p = @"quicoteto";
		_client = [[SGClient alloc] initWithUsername:u password:p];
	}

	if (!_context) {
		_context = [_client context];
		_context.delegate = self;
	}
	
	_didCallDelegate = NO;
}

- (void)tearDown
{
	[super tearDown];
}

- (void)wait
{
	NSRunLoop *l = [NSRunLoop currentRunLoop];
	while (!self.didCallDelegate && [l runMode:NSDefaultRunLoopMode beforeDate:[NSDate distantFuture]]);
	STAssertTrue(self.didCallDelegate, @"Delegate didn't get called");
	self.didCallDelegate = NO;
}

- (void)connection:(NSURLConnection *)con didSucceedForResource:(SGResource *)res inContext:(SGContext *)ctx
{
	self.didCallDelegate = YES;
}

- (void)connection:(NSURLConnection *)con didFailForResource:(SGResource *)res inContext:(SGContext *)ctx withError:(NSError *)err
{
	self.didCallDelegate = YES;
}

- (void)testLoadUser
{
	SGUser *u = [[SGUser alloc] initWithContext:_context];
	u.context = _context;
	u.url = [NSURL URLWithString:@"http://127.0.0.1:3000/users/10"];

	[_context load:u];
	[self wait];

	STAssertEqualObjects(u.username, @"quicoteto", @"username should equal quicoteto");
	STAssertEqualObjects(u.tagline, @"qweqwe", @"tagline should equal qweqwe");
	STAssertEqualObjects(u.pictureURL, [NSURL URLWithString:@"medium/default_image.png"], @"picture_url should equal medium/default_image.png");
	STAssertTrue([u.pictureURL isKindOfClass:[NSURL class]], @"pictureURL should be a NSURL");
	STAssertTrue([u.styles isKindOfClass:[SGStylesCollection class]], @"styles should be a SGStylesCollection");
}

- (void)testLoadStyle
{
	SGStyle *s = [[SGStyle alloc] initWithContext:_context];
	s.context = _context;
	s.url = [NSURL URLWithString:@"http://localhost:3000/styles/5-b81efcba888a91f4f530cd0665229ddf"];

	[_context load:s];
	[self wait];

	STAssertEqualObjects(s.thumbnailURL, [NSURL URLWithString:@"http://s3-eu-west-1.amazonaws.com/stylegeist-production/pictures/5/thumb.jpg?1364424591"], @"thumbnailURL should equal");
	STAssertTrue([s.pictureURL isKindOfClass:[NSURL class]], @"pictureURL should be a NSURL");
	STAssertTrue([s.user isKindOfClass:[SGUser class]], @"user should be a SGUser");
	STAssertTrue([s.next isKindOfClass:[SGStyle class]], @"next should be a SGStyle");
}

- (void)testLoadComment
{
	SGComment *c = [[SGComment alloc] initWithContext:_context];
	c.url = [NSURL URLWithString:@"http://localhost:3000/users/10-quicoteto/styles/4-fda2293f02933dfcaf470aadf2745089/comments/6"];

	[_context load:c];
	[self wait];

	STAssertEqualObjects(c.content, @"And another", @"thumbnailURL should equal");
	STAssertTrue([c.user isKindOfClass:[SGUser class]], @"user should be a SGUser");
	STAssertEqualObjects(c.user.username, @"quicoteto", @"username should be quicoteto");
}

- (void)testLoadRating
{
	SGRating *r = [[SGRating alloc] initWithContext:_context];
	r.url = [NSURL URLWithString:@"http://localhost:3000/users/10-quicoteto/styles/4-fda2293f02933dfcaf470aadf2745089/ratings.json"];

	[_context load:r];
	[self wait];

	STAssertEqualObjects(r.average, @4, @"rating should be 4");
	STAssertEqualObjects(r.own, @4, @"own should be 4");
}

- (void)testLoadStylesCollection
{
	SGStylesCollection *s = [[SGStylesCollection alloc] initWithContext:_context];
	s.url = [NSURL URLWithString:@"http://localhost:3000/users/10-quicoteto/styles.json"];

	[_context load:s];
	[self wait];

	STAssertTrue([s.total isKindOfClass:[NSNumber class]], @"total should be a NSNumber");
}

- (void)testLoadNextStylesCollection
{
	SGStylesCollection *u = [[SGStylesCollection alloc] initWithContext:_context];
	u.url = [NSURL URLWithString:@"http://localhost:3000/styles"];
	[_context load:u];
	[self wait];

	SGStylesCollection *next = (SGStylesCollection *)[u next];
	if (!next) {
		return;
	}
	[_context load:next];
	[self wait];

	STAssertTrue(u.total.intValue == next.total.intValue, @"total number should be equal accross pages");
	STAssertTrue(u.total.intValue == u.styles.count + next.styles.count, @"the sum of the styles in both pages should be equal to the total");
}

- (void)testLoadCommentsCollection
{
	SGCommentsCollection *c = [[SGCommentsCollection alloc] initWithContext:_context];
	c.url = [NSURL URLWithString:@"http://localhost:3000/users/10-quicoteto/styles/4-fda2293f02933dfcaf470aadf2745089/comments.json"];

	[_context load:c];
	[self wait];

	STAssertTrue([c.total isKindOfClass:[NSNumber class]], @"total should be a NSNumber");

}

- (void)testLoadNextCommentsCollection
{
	SGCommentsCollection *u = [[SGCommentsCollection alloc] initWithContext:_context];
	u.url = [NSURL URLWithString:@"http://localhost:3000/users/10-quicoteto/styles/4-fda2293f02933dfcaf470aadf2745089/comments.json"];
	[_context load:u];
	[self wait];

	SGCommentsCollection *next = (SGCommentsCollection *)[u next];
	if (!next) {
		return;
	}

	[_context load:next];
	[self wait];

	STAssertTrue(u.total.intValue == next.total.intValue, @"total number should be equal accross pages");
	STAssertTrue(u.total.intValue == u.comments.count + next.comments.count, @"the sum of the users in both pages should be equal to the total");
}

- (void)testLoadUsersCollection
{
	SGUsersCollection *u = [[SGUsersCollection alloc] initWithContext:_context];
	u.url = [NSURL URLWithString:@"http://localhost:3000/users"];

	[_context load:u];
	[self wait];

	STAssertTrue([u.total isKindOfClass:[NSNumber class]], @"total should be a NSNumber");
}

- (void)testLoadNextUsersCollection
{
	SGUsersCollection *u = [[SGUsersCollection alloc] initWithContext:_context];
	u.url = [NSURL URLWithString:@"http://localhost:3000/users"];
	[_context load:u];
	[self wait];

	SGUsersCollection *next = (SGUsersCollection *)[u next];
	if (!next) {
		return;
	}

	[_context load:next];
	[self wait];

	STAssertTrue(u.total.intValue == next.total.intValue, @"total number should be equal accross pages");
	STAssertTrue(u.total.intValue == u.users.count + next.users.count, @"the sum in the styles in both pages should be equal to the total");
}

@end